#include <windows.h>
#include <stdio.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include "tga.h"

HGLRC		hRC  = NULL;
HDC			hDC  = NULL;
HWND		hWnd = NULL;
HINSTANCE	hInstance;

bool keys[256];
bool active = true;		//window active flag

BOOL light = false;		// Light on/off
BOOL lp;							
BOOL fp;

GLfloat xrot;
GLfloat yrot;
GLfloat xspeed;
GLfloat yspeed;

GLfloat LightAmbient[]= { 0.5f, 0.5f, 0.5f, 1.0f }; 
GLfloat LightDiffuse[]= { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat LightPosition[]= { 0.0f, 0.0f, 2.0f, 1.0f };	


unsigned int texture;

LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);

GLvoid ReSizeGLScene(GLsizei width, GLsizei height) {
  if (height == 0) {
    height = 1;
  }
	glViewport(0,0,width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f, (GLfloat)width/(GLfloat)height, 0.1f, 100.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

int InitGL() {
	glShadeModel(GL_SMOOTH);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glEnable(GL_TEXTURE_2D);
	glLightfv(GL_LIGHT1, GL_AMBIENT, LightAmbient);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, LightDiffuse);
	glLightfv(GL_LIGHT1, GL_POSITION,LightPosition);
	glEnable(GL_LIGHT1);
	TGAFILE tgaBild;
	if (!LoadTGAFile("lmu.tga",&tgaBild)) return FALSE;
	glGenTextures(1,&texture);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,tgaBild.imageWidth, tgaBild.imageHeight,0,GL_RGB,GL_UNSIGNED_BYTE, tgaBild.imageData);
	return TRUE;
}


//#####################

int DrawGLScene(GLvoid) {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	glPushMatrix();
	glTranslatef(0.0f,0.0f,-6.0f);
	glRotatef(xrot,1.0f,0.0f,0.0f);		// Rotate On The X Axis By xrot
	glRotatef(yrot,0.0f,1.0f,0.0f);	
	glBindTexture(GL_TEXTURE_2D, texture);
	glColor3f(1.0f, 1.0f, 1.0f);
	glBegin(GL_QUADS);											  
	// Front Face
		glNormal3f( 0.0f, 0.0f, 1.0f);					// Normal Pointing Towards Viewer
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);	// Point 1 (Front)
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);	// Point 2 (Front)
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);	// Point 3 (Front)
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);	// Point 4 (Front)
		// Back Face
		glNormal3f( 0.0f, 0.0f,-1.0f);					// Normal Pointing Away From Viewer
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f, -1.0f);	// Point 1 (Back)
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);	// Point 2 (Back)
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);	// Point 3 (Back)
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f);	// Point 4 (Back)
		// Top Face
		glNormal3f( 0.0f, 1.0f, 0.0f);					// Normal Pointing Up
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);	// Point 1 (Top)
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f,  1.0f,  1.0f);	// Point 2 (Top)
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f,  1.0f,  1.0f);	// Point 3 (Top)
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);	// Point 4 (Top)
		// Bottom Face
		glNormal3f( 0.0f,-1.0f, 0.0f);					// Normal Pointing Down
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f, -1.0f, -1.0f);	// Point 1 (Bottom)
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f, -1.0f, -1.0f);	// Point 2 (Bottom)
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);	// Point 3 (Bottom)
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);	// Point 4 (Bottom)
		// Right face
		glNormal3f( 1.0f, 0.0f, 0.0f);					// Normal Pointing Right
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f);	// Point 1 (Right)
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);	// Point 2 (Right)
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);	// Point 3 (Right)
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);	// Point 4 (Right)
		// Left Face
		glNormal3f(-1.0f, 0.0f, 0.0f);					// Normal Pointing Left
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f, -1.0f);	// Point 1 (Left)
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);	// Point 2 (Left)
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);	// Point 3 (Left)
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);	// Point 4 (Left)
	glEnd();	
	glPopMatrix();// Done Drawing The Quad
	xrot+=xspeed;								// Add xspeed To xrot
	yrot+=yspeed;
	return TRUE;	

}

//#####################
GLvoid KillGLWindow(GLvoid) {
  if (hRC) {
		if (!wglMakeCurrent(NULL,NULL)) {
			MessageBox(NULL,"Release of DeviceContect and RenderContext failed!", "SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		}
		hRC=NULL;
  }
	if (hDC && !ReleaseDC(hWnd,hDC)) {
		MessageBox(NULL,"Release of DeviceContect", "SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hDC = NULL;
	}
	if (hWnd && !DestroyWindow(hWnd)) {
		MessageBox(NULL,"Could not release hWnd", "SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hWnd = NULL;
	}
	if (!UnregisterClass("OpenGL", hInstance)) {
		MessageBox(NULL,"Could not unregister Class", "SHUTDOWN ERROR",MB_OK | MB_ICONINFORMATION);
		hInstance = NULL;
	}
}

BOOL CreateGLWindow(char* title, int width, int height) {
	GLuint PixelFormat;	// Pixel Format Mode
	WNDCLASS wc;

	DWORD	dwExStyle;		// Window Extended Style
	DWORD dwStyle;			// Window Style

	RECT WindowRect;
	WindowRect.left   = (long)0;
	WindowRect.right  = (long)width;
  WindowRect.top    = (long)0;
	WindowRect.bottom = (long)height;

	hInstance					= GetModuleHandle(NULL);									// Grab An Instance For Our Window
	wc.style					= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;			// Redraw On Move, And Own DC For Window
	wc.lpfnWndProc		= (WNDPROC) WndProc;						  				// WndProc Handles Messages
	wc.cbClsExtra			= 0;																			// No Extra Window Data
	wc.cbWndExtra			= 0;																	  	// No Extra Window Data
	wc.hInstance			= hInstance;															// Set The Instance
	wc.hIcon					= LoadIcon(NULL, IDI_WINLOGO);						// Load The Default Icon
	wc.hCursor				= LoadCursor(NULL, IDC_ARROW);						// Load The Arrow Pointer
	wc.hbrBackground	= NULL;																	  // No Background Required For GL
	wc.lpszMenuName		= NULL;																	  // We Don't Want A Menu
	wc.lpszClassName	= "OpenGL";															  // Set The Class Name
	
	if (!RegisterClass(&wc)) {
		MessageBox(NULL,"Failed to register the window class!", "ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;
	}
	dwExStyle=WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;			// Window Extended Style
	dwStyle=WS_OVERLAPPEDWINDOW;		
	AdjustWindowRectEx(&WindowRect, dwStyle, FALSE, dwExStyle);		//extend Window to match exact width/height
	if (!(hWnd=CreateWindowEx(	dwExStyle,												// Extended Style For The Window
															"OpenGL",													// Class Name
															title,														// Window Title
															WS_CLIPSIBLINGS |									// Required Window Style
															WS_CLIPCHILDREN |									// Required Window Style
															dwStyle,													// Selected Window Style
															0, 0,															// Window Position
															WindowRect.right-WindowRect.left,	// Calculate Adjusted Window Width
															WindowRect.bottom-WindowRect.top,	// Calculate Adjusted Window Height
															NULL,															// No Parent Window
															NULL,															// No Menu
															hInstance,												// Instance
															NULL)))	{													// Don't Pass Anything To WM_CREATE
		
			KillGLWindow();																						// Reset The Display
			MessageBox(NULL,"Window Creation Error.","ERROR",MB_OK|MB_ICONEXCLAMATION);
			return FALSE;																							// Return FALSE
	}
	static	PIXELFORMATDESCRIPTOR pfd=					// pfd Tells Windows How We Want Things To Be
		{
			sizeof(PIXELFORMATDESCRIPTOR),					// Size Of This Pixel Format Descriptor
			1,																			// Version Number
			PFD_DRAW_TO_WINDOW |										// Format Must Support Window
			PFD_SUPPORT_OPENGL |										// Format Must Support OpenGL
			PFD_DOUBLEBUFFER,												// Must Support Double Buffering
			PFD_TYPE_RGBA,													// Request An RGBA Format
			32,																		// Select Our Color Depth
			0, 0, 0, 0, 0, 0,												// Color Bits Ignored
			0,																			// No Alpha Buffer
			0,																			// Shift Bit Ignored
			0,																			// No Accumulation Buffer
			0, 0, 0, 0,															// Accumulation Bits Ignored
			16,																			// 16Bit Z-Buffer (Depth Buffer)
			0,																			// No Stencil Buffer
			0,																			// No Auxiliary Buffer
			PFD_MAIN_PLANE,													// Main Drawing Layer
			0,																			// Reserved
			0, 0, 0																	// Layer Masks Ignored
	};
	if (!(hDC=GetDC(hWnd))) {
		KillGLWindow();
		MessageBox(NULL, "Can't create a GL Device Context", "ERROR", MB_OK|MB_ICONEXCLAMATION);
		return FALSE;
	}
	
	if (!(PixelFormat=ChoosePixelFormat(hDC,&pfd)))	{			// Did Windows Find A Matching Pixel Format?
		KillGLWindow();																			// Reset The Display
		MessageBox(NULL,"Can't Find A Suitable PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;																				// Return FALSE
	}
	if(!SetPixelFormat(hDC,PixelFormat,&pfd))	{			// Are We Able To Set The Pixel Format?
		KillGLWindow();																// Reset The Display
		MessageBox(NULL,"Can't Set The PixelFormat.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;																	// Return FALSE
	}
	if (!(hRC=wglCreateContext(hDC))) {					// Are We Able To Get A Rendering Context?
		KillGLWindow();														// Reset The Display
		MessageBox(NULL,"Can't Create A GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;															// Return FALSE
	}
	if(!wglMakeCurrent(hDC,hRC)) {						// Try To Activate The Rendering Contex
		KillGLWindow();													// Reset The Display
		MessageBox(NULL,"Can't Activate The GL Rendering Context.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;														// Return FALSE
	}
	ShowWindow(hWnd,SW_SHOW);						// Show The Window
	SetForegroundWindow(hWnd);						// Slightly Higher Priority
	SetFocus(hWnd);											// Sets Keyboard Focus To The Window
	ReSizeGLScene(width, height);						// Set Up Our Perspective GL Screen
	
	if (!InitGL()) {								// Initialize Our Newly Created GL Window
		KillGLWindow();							// Reset The Display
		MessageBox(NULL,"Initialization Failed.","ERROR",MB_OK|MB_ICONEXCLAMATION);
		return FALSE;							// Return FALSE
	}
	return TRUE;
}

LRESULT CALLBACK WndProc(	HWND	hWnd,
													UINT	uMsg,										// Message For This Window
													WPARAM	wParam,								// Additional Message Information
													LPARAM	lParam) 								// Additional Message Information
{
	switch(uMsg)
	{
		case WM_ACTIVATE:								// Watch For Window Activate Message
		{
			if (!HIWORD(wParam))					// Check Minimization State
			{
				active=TRUE;								// Program Is Active
			}
			else
			{
				active=FALSE;								// Program Is No Longer Active
			}

			return 0;											// Return To The Message Loop
		}
		case WM_SYSCOMMAND:						// Intercept System Commands
		{
			switch (wParam)							// Check System Calls
			{
				case SC_SCREENSAVE:				// Screensaver Trying To Start?
				case SC_MONITORPOWER:			// Monitor Trying To Enter Powersave?
				return 0;									// Prevent From Happening
			}
			break;											// Exit
		}
		case WM_CLOSE:								// Did We Receive A Close Message?
		{
			PostQuitMessage(0);					// Send A Quit Message
			return 0;										// Jump Back
		}
		case WM_KEYDOWN:						// Is A Key Being Held Down?
		{
			keys[wParam] = TRUE;			// If So, Mark It As TRUE
			return 0;									// Jump Back
		}
		case WM_KEYUP:							// Has A Key Been Released?
		{
			keys[wParam] = FALSE;					// If So, Mark It As FALSE
			return 0;						// Jump Back
		}
		case WM_SIZE:							// Resize The OpenGL Window
		{
			ReSizeGLScene(LOWORD(lParam),HIWORD(lParam));		// LoWord=Width, HiWord=Height
			return 0;																				// Jump Back
		}
	}
	// Pass All Unhandled Messages To DefWindowProc
	return DefWindowProc(hWnd,uMsg,wParam,lParam);
}

int WINAPI WinMain(	HINSTANCE	hInstance,				// Instance
										HINSTANCE	hPrevInstance,		// Previous Instance
										LPSTR			lpCmdLine,	  		// Command Line Parameters
										int				nCmdShow)			  	// Window Show State
{
	MSG msg;
	BOOL done=FALSE;
  if (!CreateGLWindow("MIMUC",640,480))
	{
		return 0;							// Quit If Window Was Not Created
	}
	// Main Message Loop
	while(!done) {
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE)) {
			if (msg.message==WM_QUIT) {
				done = TRUE;
			}
			else {
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
		}
		else {
		// Draw The Scene.  Watch For ESC Key And Quit Messages From DrawGLScene()
			if (active)						// Program Active?
			{
				if (keys[VK_ESCAPE])				// Was ESC Pressed?
				{
					done=TRUE;				// ESC Signalled A Quit
				}
				else						// Not Time To Quit, Update Screen
				{
					DrawGLScene();				// Draw The Scene
					SwapBuffers(hDC);			// Swap Buffers (Double Buffering)
				}
			}
			if (keys[VK_UP]) {
				xspeed-=0.01f;
			}
			if (keys['L'] && !lp) {
				lp = true;
				light = !light;
				if (!light) {
				  glDisable(GL_LIGHTING);
					}
				else {
					glEnable(GL_LIGHTING);
					};
			}
			if (!keys['L'])					// Has L Key Been Released?
				{
					lp=FALSE;				// If So, lp Becomes FALSE
				}

			if (keys[VK_DOWN]) {
				xspeed+=0.01f;
			}
			if (keys[VK_RIGHT]) {
				yspeed+=0.01f;
			}
			if (keys[VK_LEFT]) {
				yspeed-=0.01f;
			}
      if (keys[VK_F1])					// Is F1 Being Pressed?
			{
				keys[VK_F1]=FALSE;				// If So Make Key FALSE
				KillGLWindow();					// Kill Our Current Window
				// Recreate Our OpenGL Window
				if (!CreateGLWindow("MIMUC",640,480))
				{
					return 0;				// Quit If Window Was Not Created
				}
			}
		}
	}
	// Shutdown
	KillGLWindow();								// Kill The Window
	return (int)(msg.wParam);							// Exit The Program
}








		



